<?php
/*
 * $Horde: imp/mailbox.php,v 2.270 2001/05/29 13:32:13 chuck Exp $
 *
 * Copyright 1999, 2000, 2001 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000, 2001 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/*
 * Check the user's address and the list of addresses for which To: should
 * be displayed.
 */
function display_toaddress($user, $server, $from) {
    return is_int(strpos($from, IMP::getPlainAddress($user, $server)));
}

function runSearch (&$sorted) {
    global $imp, $folders, $prefs;

    $sorted = array();
    if (empty($imp['searchquery']) || empty($imp['searchfolders'])) {
        return array();
    }

    for ($i = 0; $i < count($imp['searchfolders']); $i++) {
        if (strstr($imp['protocol'], 'pop3')) {
            $stream = $imp['stream'];
        } else {
            $stream = @imap_open(IMP::serverString() . $imp['searchfolders'][$i],
                                 $imp['user'], Secret::read(Secret::getKey('imp'), $imp['pass']), OP_READONLY);
        }

        if ($stream) {
            $results = imap_sort($stream, $prefs->getValue('sortby'),
                                 $prefs->getValue('sortdir'), SE_UID, $imp['searchquery']);
            if (is_array($results)) {
                for ($j = 0; $j < count($results); $j++) {
                    $sorted[] = $results[$j];
                    $folders[] = $imp['searchfolders'][$i];
                }
            }
            if (!strstr($imp['protocol'], 'pop3')) {
                imap_close($stream);
            }
        }
    }

    if (isset($folders)) {
        $imp['messagefolders'] = implode(':', $folders);
        return implode(':', $sorted);
    } else {
        return false;
    }
}

function stylize($string, $styles)
{
    if (!is_array($styles)) return $string;

    foreach ($styles as $style) {
        if (!empty($style)) {
            $string = '<' . $style . '>' . $string . '</' . $style . '>';
        }
    }

    return $string;
}

$set = false;
if (isset($new_lang)) {
    $language = $new_lang;
    $set = true;
}

define('IMP_BASE', dirname(__FILE__));
require_once IMP_BASE . '/lib/base.php';
require_once IMP_BASE . '/lib/Message.php';
require_once HORDE_BASE . '/lib/MIME.php';
require IMP_BASE . '/config/mime.php';

session_name($conf['session_name']);
@session_start();

/* Initialize $actionID to a resonable value if it hasn't already been set. */
$actionID = Horde::getFormData('actionID', NO_ACTION);

if (!(IMP::setupSession() &&
      IMP::authenticate())) {
    header('Location: ' . Horde::applicationUrl(IMP::logoutUrl('login.php?reason=failed'), true));
    exit;
}

/* Massage data coming in via GET/POST to be where we expect it. */
if (isset($HTTP_POST_VARS['indices'])) {
    $indices = $HTTP_POST_VARS['indices'];
    if (!is_array($indices)) $indices = array($indices);
} elseif (isset($HTTP_GET_VARS['indices'])) {
    $indices = $HTTP_GET_VARS['indices'];
    if (!is_array($indices)) $indices = array($indices);
}
if (isset($HTTP_POST_VARS['targetMbox'])) {
    $targetMbox = $HTTP_POST_VARS['targetMbox'];
} elseif (isset($HTTP_GET_VARS['targetMbox'])) {
    $targetMbox = $HTTP_GET_VARS['targetMbox'];
}

/* If this is a login, and the user wants to run filters on login,
   make sure they get run. */
if ($actionID == IMP_LOGIN && $prefs->getValue('filter_on_login')) {
    $actionID = FILTER;
}

/* Set the current time zone. */
if ($prefs->getValue('timezone') != '') {
    putenv('TZ=' . $prefs->getValue('timezone'));
}

/* Run through the action handlers */
switch ($actionID) {
 case NO_ACTION:
     break;

 case FILTER:
     $filters = @unserialize($prefs->getValue('filters'));
     if (is_array($filters) && count($filters) && !strstr($imp['protocol'], 'pop3')) {
         include_once IMP_BASE . '/lib/Folder.php';
         $baseQuery = 'UNDELETED ';
         for ($i = 0; $i < count($filters); $i++) {
             for ($j = 0; $j < count($filters[$i]['fields']); $j++) {
                 $query = $baseQuery . strtoupper($filters[$i]['fields'][$j]) . ' "' . $filters[$i]['text'] . '"';
                 $indices = imap_search($imp['stream'], $query, SE_UID);
                 if (isset($indices) && is_array($indices)) {
                     if ($filters[$i]['action'] == 'delete') {
                         IMP_Message::delete($indices);
                     }
                     else {
                         if (IMP_Folder::exists($imp['stream'], $filters[$i]['folder'])) {
                             IMP_Message::copy($filters[$i]['folder'], $indices, MOVE_MESSAGES);
                         } else {
                             Horde::raiseMessage(_(sprintf("%s %d: %s: ", "problem with filter rule", $i + 1, "folder does not exist")) . IMP::displayFolder($filters[$i]['folder']), HORDE_WARNING);
                         }
                     }
                 }
             }
         }
     }
     break;
}


$msgs = imap_search($imp['stream'], 'UNSEEN');
$imp['msgcount'] = ($msgs !== false) ? count($msgs) : 0;

if ($imp['msgcount'] != 0) {
        $sorted = imap_sort($imp['stream'], $prefs->getValue('sortby'),
                            $prefs->getValue('sortdir'), SE_UID, 'UNSEEN');
} else {
    $sorted = array();
}

$begin = 1;
$end = $imp['msgcount'];
$page = 1;
if ($imp['msgcount'] > $prefs->getValue('max_msgs')) {
    $url = Horde::applicationUrl('mailbox.php?page=1');
}

if ($imp['mailbox'] == '**search') {
    $imp['offset'] = 0;
    $beginIndex = $begin - 1;
} else {
    $imp['msgl'] = implode(':', IMP_Message::range($sorted, $begin, $offset, $beginIndex, $num));
    $imp['offset'] = $offset;
}

require $conf['paths']['templates'] . '/portal/header.inc';
if ($imp['mailbox'] != '**search') {
    include $conf['paths']['templates'] . '/portal/message_headers.inc';
}


// now step through each message
$newmsgs = 0;
for ($i = $begin, $j = $beginIndex; $i <= $end; $i++, $j++) {

    /* Make sure that the index is actually in the slice of messages
       we're looking at. If we're hiding deleted messages, for
       example, there may be gaps here. */
    if (isset($sorted[$i - 1])) {
        $msgnum = $sorted[$i - 1];

        $h = @imap_header($imp['stream'], imap_msgno($imp['stream'], $msgnum));
        if (!$h) {
            // FIXME: Log something to info-level log here
            continue;
        }

        /* initialize the header fields */
        $dat = '';
        $frm = '';
        $fullfrom = '';
        $to = '';
        $sub = '';
        $msg_size = '?';

        /* Now pull the IMAP header values into them, decoding them at
           the same time. */

        /* formats the header date string nicely */
        if (isset($h->date)) {
            $udate = strtotime($h->date);
            if ((date('Y') != date('Y', $udate)) ||
                (date('M') != date('M', $udate)) ||
                (date('d') != date('d', $udate))) {
                // not today, use the date
                $dat = strftime($conf['mailbox']['date_format'], $udate);
            } else {
                // else it's today, use the time
                $dat = strftime($conf['mailbox']['time_format'], $udate);
            }
        } else {
            $dat = '';
        }

        if (isset($h->from[0])) {
            $fullfrom = '';

            if (isset($h->from[0]->personal)) {
                $fullfrom = $frm = MIME::decode($h->from[0]->personal);
            }

            if (isset($h->from[0]->mailbox) && isset($h->from[0]->host)) {
                if (empty($frm)) $frm = $h->from[0]->mailbox . '@' . $h->from[0]->host;
                $fullfrom .= $h->from[0]->mailbox . '@' . $h->from[0]->host;
            }
        }

        if (display_toaddress($imp['user'], $imp['server'], $fullfrom)) {
            if (isset($h->to[0])) {
                if (isset($h->to[0]->personal)) {
                    $to = MIME::decode($h->to[0]->personal);
                }
                if (empty($to) && isset($h->to[0]->mailbox) && isset($h->to[0]->host)) {
                    $to = $h->to[0]->mailbox . '@' . $h->to[0]->host;
                }
            } else {
                $to = _("Undisclosed Recipients");
            }
            $frm = _("To") . ': ' . $to;
        }

        if (isset($h->subject)) {
            $sub = MIME::decode($h->subject);
        }
        if (isset($h->Size)) {
            $msg_size = ($h->Size > 1024)
                ? sprintf('%.0fkb', $h->Size / 1024)
                : $h->Size;
        }
        if (strlen($frm) > $conf['mailbox']['max_from_chars']) {
            $frm = substr($frm, 0, $conf['mailbox']['max_from_chars']) . '...';
        }
        if (strlen($sub) > $conf['mailbox']['max_subj_chars']) {
            $sub = substr($sub, 0, $conf['mailbox']['max_subj_chars']) . '...';
        }

        /* make sure that they get put into an HTML readable format */
        $frm = htmlspecialchars($frm);
        $sub = htmlspecialchars($sub);

        /* set them to useful things for IMP if they ended up being empty */
        if ($dat == '') { $dat = '&nbsp;'; }
        if ($frm == '') { $frm = '&nbsp;'; }
        if ($sub == '') { $sub = '[' . _("No subject") . ']'; }

        /* filter the subject text, if requested */
        if ($prefs->getValue('filtering')) {
            include_once HORDE_BASE . '/lib/Text.php';
            $sub = Text::filter($sub, $conf['msg']['filtering']['words'],
                                $conf['msg']['filtering']['replacement']);
        }

        $attachment = false;
        $structure = @imap_fetchstructure($imp['stream'], $msgnum, FT_UID);
        if (is_object($structure)) {
            if (!empty($structure->subtype)) {
                if (strtolower($structure->subtype) == 'signed' ||
                    strtolower($structure->subtype) == 'encrypted') {
                    $attachment = 'signed';
                } elseif (strtolower($structure->subtype) == 'alternative') {
                    $attachment = 'alternative';
                } elseif (isset($structure->parts) &&
                          is_array($structure->parts) &&
                          count($structure->parts) > 0) {
                    $attachment = 'attachment';
                } elseif ((!empty($structure->type) &&
                           $structure->type != TYPETEXT) ||
                          !empty($structure->subtype) &&
                          strtolower($structure->subtype) != 'plain') {
                    $attachment = 'attachment';
                }
            }
        }

        if ($imp['mailbox'] == '**search') {
            $target = Horde::applicationUrl('message.php?mailbox=**search&msgmailbox=' .
                                 urlencode($folders[$i-1]) . '&index=' . urlencode($msgnum) .
                                 '&array_index=' . $j);
        } else {
            $target = Horde::applicationUrl('message.php?index=' . urlencode($msgnum) .
                                 '&array_index=' . $j);
        }

        /* get all the flag information */
        $flagbits = 0;
        $style = array();
        $status = '';
        $bg = 'text';
        if (!strstr($imp['protocol'], 'pop3')) {
            if (!empty($h->toaddress) && strstr(strtolower($h->toaddress), strtolower(IMP::getPlainAddress()))) {
                $status .= $conf['mailbox']['personal_flag'];
                $flagbits |= IMP_PERSONAL;
            }
            if ($h->Recent == 'R' || $h->Recent == 'N') {
                $newmsgs++;
            }
            if ($h->Unseen == 'U' || $h->Recent == 'N') {
                $flagbits |= IMP_UNSEEN;
                $status .= $conf['mailbox']['unseen_flag'];
                $style[] = $conf['mailbox']['unseen_style'];
                $bg = 'unseen';
            }
            if ($h->Answered == 'A') {
                $flagbits |= IMP_ANSWERED;
                $status .= $conf['mailbox']['answered_flag'];
                $style[] = $conf['mailbox']['answered_style'];
                $bg = 'answered';
            }
            if (isset($h->Draft) && $h->Draft == 'X') {
                $flagbits |= IMP_DRAFT;
                $status .= $conf['mailbox']['draft_flag'];
                $style[] = $conf['mailbox']['draft_style'];
                $target = IMP::composeLink(array(), array('actionID' => DRAFT, 'mailbox' => $imp['mailbox'], 'index' => $msgnum, 'bodypart' => 1));
            }
            if ($h->Flagged == 'F') {
                $flagbits |= IMP_FLAGGED;
                $status .= $conf['mailbox']['important_flag'];
                $style[] = $conf['mailbox']['important_style'];
                $bg = 'important';
            }
            if ($h->Deleted == 'D') {
                $flagbits |= IMP_DELETED;
                $status .= $conf['mailbox']['deleted_flag'];
                $style[] = $conf['mailbox']['deleted_style'];
                $bg = 'deleted';
            }
            $flags[] = $flagbits;
        } else {
            $flags[] = 0;
        }

        // apply styles
        $styledat = stylize($dat, $style);
        $stylefrm = stylize($frm, $style);
        $stylesub = stylize($sub, $style);

        // decide what to do in the from column (after we've possibly
        // applied styles to $from)
        if ($conf['mailbox']['from_link'] == 'message') {
            if ($imp['mailbox'] == '**search') {
                $from_uri = Horde::applicationUrl('message.php?mailbox=**search&msgmailbox=' .
                                                  urlencode($folders[$i-1]) . '&index=' . urlencode($msgnum) .
                                                  '&array_index=' . $j);
            } else {
                $from_uri = Horde::applicationUrl("message.php?index=$msgnum&array_index=$j");
            }
            $from_link = Horde::link($from_uri, $frm) . $stylefrm . '</a>';
        } elseif ($conf['mailbox']['from_link'] == 'compose') {
            $compose_url = IMP::composeLink(array(), array('actionID' => MAILTO, 'mailbox' => $imp['mailbox'], 'index' => $msgnum));
            $from_link = Horde::link($compose_url, sprintf(_("Compose Message (%s)"), $frm)) . "$stylefrm</a>";
        } else {
            $from_link = $stylefrm;
        }

        include $conf['paths']['templates'] . '/portal/message_summaries.inc';
    }
} // end message display loop

if ($end == 0) {
    if ($imp['mailbox'] == '**search') {
        include $conf['paths']['templates'] . '/portal/message_headers.inc';
    }
    include $conf['paths']['templates'] . '/portal/empty_mailbox.inc';
}

require $conf['paths']['templates'] . '/portal/message_footers.inc';
require $conf['paths']['templates'] . '/portal/footer.inc';

$registry->shutdown();
if (!empty($open_compose_window)) {
    $registry->pinvoke('mail/compose', array());
}

$prefs->store();

?>
